require 'installer/action'
require 'installer/errors'

class Installer::Planner::Step::PurgePE < Installer::Planner::Step
  def initialize(infra, data={}, pre=nil, post=nil)
    super

    @host = @infra.hosts[data[:host]] if data[:host]
    @local_installer = '/opt/puppet/share/installer'
    @local_inst = @local_installer + '/installer/'
  end

  def perform_step(&execute)
    mktmp = Installer::Action::Execute.new(:hosts => [@host], :cmd => "mktemp -d -t pe-installer-XXXXXXXX")
    yield mktmp
    unless mktmp.success?
      error(@host.hostname, "Failed to create temporary directory for PE installation: #{mktmp.results}")
      return
    end

    # TODO: Some parsing for the tmp dir? Define these better
    tmpdir = Pathname.new(mktmp.results[@host.hostname][:stdout].strip)

    # Ensure there is an installer available to copy
    unless File.readable? "#{@local_inst}/puppet-enterprise-uninstaller"
      error("localhost", "PE Un-Installer not found in directory #{@local_inst}")
      return
    end

    # Copy uninstaller files
    uninstaller = Installer::Action::Upload.new(:to_host => @host, :from => "#{@local_inst}/puppet-enterprise-uninstaller", :to => "#{tmpdir}/puppet-enterprise-uninstaller")
    yield uninstaller
    utils = Installer::Action::Upload.new(:to_host => @host, :from => "#{@local_inst}/utilities", :to => "#{tmpdir}/utilities")
    yield utils
    unless uninstaller.success? && utils.success?
      error(@host.hostname, "Failed to copy uninstaller to remote host #{@host.hostname} for PE installation")
      return
    end

    uninstall = Installer::Action::Execute.new(:host => @host, :sudo => true, :cmd => "#{tmpdir}/puppet-enterprise-uninstaller -ydp")
    yield uninstall
    unless uninstall.success?
      error(@host.hostname, "Puppet Enterprise uninstall failed.")
      return
    end
  end

  def name
    :uninstall_pe
  end

  def description
    "Uninstall and purge all traces of PE."
  end
end
